@@ -18,6 +18,6 @@ def get_supported_dcv_os(architecture):
     """Return a list of all the operating system supported by DCV."""
     architectures_dict = {
         "x86_64": SUPPORTED_OSS,
-        "arm64": ["ubuntu1804", "alinux2", "centos7", "centos8"],
+        "arm64": ["ubuntu1804", "alinux2", "centos7"],
     }
     return architectures_dict.get(architecture, [])
