@@ -8,15 +8,8 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
-# fmt: off
-from __future__ import absolute_import, print_function  # isort:skip
 
 import functools
-
-from future import standard_library  # isort:skip
-standard_library.install_aliases()
-# fmt: on
-
 import hashlib
 import json
 import logging
@@ -141,8 +134,8 @@ def generate_random_name_with_prefix(name_prefix):
 
     Example: <name_prefix>-4htvo26lchkqeho1
     """
-    random_string = "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(16))
-    output_name = "-".join([name_prefix.lower()[: 63 - len(random_string) - 1], random_string])
+    random_string = "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(16))  # nosec
+    output_name = "-".join([name_prefix.lower()[: 63 - len(random_string) - 1], random_string])  # nosec
     return output_name
 
 
@@ -562,27 +555,19 @@ def get_supported_os_for_scheduler(scheduler):
     :param scheduler: the scheduler for which we want to know the supported os
     :return: an array of strings of the supported OSes
     """
-    oses = ["alinux", "alinux2"]
+    oses = ["alinux2"]
     if scheduler != "awsbatch":
-        oses.extend(["centos7", "centos8", "ubuntu1604", "ubuntu1804"])
+        oses.extend(["centos7", "ubuntu1804", "ubuntu2004"])
     return list(oses)
 
 
-def get_supported_os_for_architecture(architecture):
-    """Return list of supported OSes for the specified architecture."""
-    oses = ["alinux2", "ubuntu1804", "centos8"]
-    if architecture == "x86_64":
-        oses.extend(["centos7", "alinux", "ubuntu1604"])
-    return oses
-
-
 def get_supported_schedulers():
     """
     Return a tuple of the scheduler supported by parallelcluster.
 
     :return: a tuple of strings of the supported scheduler
     """
-    return "sge", "torque", "slurm", "awsbatch"
+    return "slurm", "awsbatch"
 
 
 def get_stack_output_value(stack_outputs, output_key):
@@ -745,12 +730,11 @@ def get_installed_version():
 def check_if_latest_version():
     """Check if the current package version is the latest one."""
     try:
-        latest = json.loads(urllib.request.urlopen("https://pypi.python.org/pypi/aws-parallelcluster/json").read())[
-            "info"
-        ]["version"]
+        pypi_url = "https://pypi.python.org/pypi/aws-parallelcluster/json"
+        latest = json.loads(urllib.request.urlopen(pypi_url).read())["info"]["version"]  # nosec nosemgrep
         if packaging.version.parse(get_installed_version()) < packaging.version.parse(latest):
             print("Info: There is a newer version %s of AWS ParallelCluster available." % latest)
-    except Exception:
+    except Exception:  # nosec
         pass
 
 
@@ -1066,21 +1050,12 @@ def cluster_has_running_capacity(stack_name):
     return cluster_has_running_capacity.cached_result
 
 
-def disable_ht_via_cpu_options(instance_type, default_threads_per_core=None):
-    """Return a boolean describing whether hyperthreading should be disabled via CPU options for instance_type."""
-    if default_threads_per_core is None:
-        default_threads_per_core = InstanceTypeInfo.init_from_instance_type(instance_type).default_threads_per_core()
-    res = all(
-        [
-            # If default threads per core is 1, HT doesn't need to be disabled
-            default_threads_per_core > 1,
-            # Currently, hyperthreading must be disabled manually on *.metal instances
-            not (
-                instance_type.endswith(".metal") or instance_type.startswith("m4.") or instance_type in ["cc2.8xlarge"]
-            ),
-        ]
-    )
-    return res
+def disable_ht_via_cpu_options(instance_type):
+    """Return a boolean describing whether hyper-threading should be disabled via CPU options for instance_type."""
+    instance_info = InstanceTypeInfo.init_from_instance_type(instance_type)
+    # If default threads per core is 1, HT doesn't need to be disabled
+    # When CpuOptions is not supported, ValidThreadsPerCore can't be found in VCpuInfo info
+    return instance_info.default_threads_per_core() > 1 and 1 in instance_info.valid_threads_per_core()
 
 
 def is_hit_enabled_scheduler(scheduler):
@@ -1103,7 +1078,7 @@ def read_remote_file(url):
             bucket, key = match.group(1), match.group(2)
             file_contents = boto3.resource("s3").Object(bucket, key).get()["Body"].read().decode("utf-8")
         else:
-            with urllib.request.urlopen(url) as f:
+            with urllib.request.urlopen(url) as f:  # nosec nosemgrep
                 file_contents = f.read().decode("utf-8")
         return file_contents
     except Exception as e:
@@ -1119,8 +1094,12 @@ def render_template(template_str, params_dict, tags, config_version=None):
     :param params_dict: Template parameters dict
     """
     try:
-        environment = Environment(loader=BaseLoader)
-        environment.filters["sha1"] = lambda value: hashlib.sha1(value.strip().encode()).hexdigest()
+        # A nosec comment is appended to the following line in order to disable the B701 check.
+        # This is done because it's needed to enable the desired functionality. The current callers
+        # of this function pass a template_str representing either a custom template specified by
+        # the user or the default template.
+        environment = Environment(loader=BaseLoader)  # nosec nosemgrep
+        environment.filters["sha1"] = lambda value: hashlib.sha1(value.strip().encode()).hexdigest()  # nosec nosemgrep
         environment.filters["bool"] = lambda value: value.lower() == "true"
         template = environment.from_string(template_str)
         output_from_parsed_template = template.render(config=params_dict, config_version=config_version, tags=tags)
@@ -1315,6 +1294,12 @@ class InstanceTypeInfo:
 
         return gpu_count
 
+    def gpu_type(self):
+        """Return name or type of the GPU for the instance."""
+        gpu_info = self.instance_type_data.get("GpuInfo", None)
+        # Remove space and change to all lowercase for name
+        return "no_gpu_type" if not gpu_info else gpu_info.get("Gpus")[0].get("Name").replace(" ", "").lower()
+
     def max_network_interface_count(self):
         """Max number of NICs for the instance."""
         needed_interfaces = int(self.instance_type_data.get("NetworkInfo").get("MaximumNetworkCards", 1))
@@ -1322,16 +1307,11 @@ class InstanceTypeInfo:
 
     def default_threads_per_core(self):
         """Return the default threads per core for the given instance type."""
-        # NOTE: currently, .metal instances do not contain the DefaultThreadsPerCore
-        #       attribute in their VCpuInfo section. This is a known issue with the
-        #       ec2 DescribeInstanceTypes API. For these instance types an assumption
-        #       is made that if the instance's supported architectures list includes
-        #       x86_64 then the default is 2, otherwise it's 1.
-        threads_per_core = self.instance_type_data.get("VCpuInfo", {}).get("DefaultThreadsPerCore")
-        if threads_per_core is None:
-            supported_architectures = self.instance_type_data.get("ProcessorInfo", {}).get("SupportedArchitectures", [])
-            threads_per_core = 2 if "x86_64" in supported_architectures else 1
-        return threads_per_core
+        return self.instance_type_data.get("VCpuInfo", {}).get("DefaultThreadsPerCore")
+
+    def valid_threads_per_core(self):
+        """Return the valid threads per core for the given instance type."""
+        return self.instance_type_data.get("VCpuInfo", {}).get("ValidThreadsPerCore", [])
 
     def vcpus_count(self):
         """Get number of vcpus for the given instance type."""
@@ -1359,3 +1339,15 @@ class InstanceTypeInfo:
             supported_classes.remove("on-demand")
             supported_classes.append("ondemand")
         return supported_classes
+
+
+@Cache.cached
+def get_fsx_info(fs_id):
+    try:
+        return boto3.client("fsx").describe_file_systems(FileSystemIds=[fs_id]).get("FileSystems")[0]
+    except ClientError as e:
+        error(
+            "Failed when retrieving FSx filesystem data for filesystem {0}: {1}".format(
+                fs_id, e.response.get("Error").get("Message")
+            )
+        )
